var assert   = require('chai').assert,
	shell    = require('shelljs');
 	jsonfile = require('jsonfile');
	sprintf  = require('sprintf-js').sprintf;

// the module we're testing
var Inventory = require('../inventory');

// setup test data
var n_test_files = 4;
var test_filenames = [];  //name of inventory files used for testing
var test_vms = [];  //expected inventory of the test file

// NOTE 
// config attr in test files have hardcoded paths
// for current test cases this is ok
// for test cases that acutaully start/stop, etc, the path must be patched

for( var i = 0; i < n_test_files; i++ ){
	test_filenames[i] = sprintf("./test/%02d", i) + "-test-vmInventory";
}


// - --------------------------------------------
// - before any test runs, load all test inventory data
// - 
before( function(done) {
	test_filenames.forEach( function(filename){
		jsonfile.readFile( filename + ".json", function(error, obj){
			if( error )
				console.log(error);	// we should halt here, bad setup.
			test_vms.push( obj );
			if( test_vms.length == test_filenames.length ){
				done();
			}
		});
	});
});

beforeEach( function() {
	backup_inventory_file();
});

afterEach( function(){
	restore_inventory_file();
});




describe("inventory", function(){

	it('should be returned without error', function(done){
		var inventory = new Inventory();
		inventory.init( function(error){
			assert(error === undefined || error == null);
			inventory.close();
			done();
		})
	})

	it('should be a list of vms', function(done){
		var inventory = new Inventory();
		inventory.init( function(error){
			assert.isArray( inventory.vms );
			assert.isTrue( inventory.vms.length > 0 );
			inventory.close();
			done();
		})
	})

	it('should contain vm object properties', function(done){
		var inventory = new Inventory();
		inventory.init( function(error){
			assert( inventory.vms.length > 0 );
			inventory.vms.forEach( function(vm){
				assert.property(vm, "config");
				assert.property(vm, "UUID");
				assert.property(vm, "DisplayName");
				assert.property(vm, "ItemID");
				assert.property(vm, "State");
			})
			inventory.close();
			done();
		})
	})

	it('should contain vm object property types', function(done){
		var inventory = new Inventory();
		inventory.init( function(error){
			assert( inventory.vms.length > 0 );
			inventory.vms.forEach( function(vm){
				assert.isString(vm.config);
				assert.isString(vm.UUID);
				assert.isString(vm.DisplayName);
				assert.isString(vm.ItemID);
				assert.isString(vm.State);
			})
			inventory.close();
			done();
		})
	})

	it('should get the correct data from the inventory file', function(done){
		var test_idx = 0;
		copy_test_inventory(test_idx);		
		var inventory = new Inventory();
		inventory.init( function(error) {
			compare_vm_lists( test_vms[test_idx], inventory.vms );
			inventory.close();
			done();
		});
	})

	// run this one a bunch of times to make sure the watcher is well behaved.
	for( var n = 0; n < 10; n++ ){

		it('should fire changed event when inventory file changes', function(done){
			this.timeout(6000);
			copy_test_inventory(0);
			var inventory = new Inventory();
			inventory.init( function(error) {
				compare_vm_lists( test_vms[0], inventory.vms );
				setTimeout( function(){copy_test_inventory(1);}, 1500 );
			});
			inventory.on('changed', function(error){
				assert(error === undefined || error == null);
				compare_vm_lists( test_vms[1], inventory.vms );
				inventory.close();
				done();
			});
		});
	}
});




describe( "vmshell", function(){

	it('should have a test case', function(){
		assert.isString( "eventually" );
	});

	it('should have a great test case', function(){
		assert.isString( "eventually" );
	});

	it('should have lots of test cases', function(){
		assert.isString( "eventually" );
	});
})



function compare_vm_lists( _vms, vms ){
	assert.lengthOf( vms, _vms.length );
	for( var i = 0; i < _vms.length; i++ ){
		var _vm = _vms[i];
		var  vm =  vms[i];
		Object.keys(_vm).forEach( function(key) {
			assert.propertyVal( vm, key, _vm[key] );
		});
	}
}

var backup_inventory_filename = Inventory.filename + ".bak";

function copy_test_inventory( idx ) {
	shell.cp("-f", test_filenames[idx], Inventory.filename);
}

function backup_inventory_file(){
	shell.cp( "-f", Inventory.filename, backup_inventory_filename );
}

function restore_inventory_file(){
	shell.cp( "-f", backup_inventory_filename, Inventory.filename );
}