// a shell to interact with vm's (mostly fusion)
// adapted from this: http://www.nodelabs.org/shell.html
var sprintf = require('sprintf-js').sprintf;
var clc = require('cli-color');
var Inventory = require('./inventory');

// commands have a name
// commands have aliases
// commands have a main function
// commands have a help string
var commands = {};


// currently selected vm is the index into inventory.vms
// since the inventory.vms list can change, in some cases
// this will either be out of range, or unexpectedly change
// your currently selected vm
// BUG: Fix
var curr = 0;

// TODO: handle changes to the inventory
var inventory = new Inventory();


// - --------------------------------------------
// - list vm's
// - 
commands['list'] = {
	aliases: ['ls'],
	help: 'list virtual machines',
	fn: function(args, finished){
		log_inventory(inventory.vms);
		finished();
	}
}


// - --------------------------------------------
// - change the currently selected vm
// - 
commands['select'] = {
	aliases: ['cd', 'cvm', 'sel'],
	help: 'select a vm by name',
	fn: function(args, finished){
		for( var i = 0; i < inventory.vms.length; i++ ){
			if( inventory.vms[i].DisplayName == args[0] ){
				curr = i;
				console.log();
				finished();
				return;
			}
		}
		log_error("unable to select unknown vm", args[0]);
		console.log();
		finished();
	}
}


// - --------------------------------------------
// - Get help
// - 
commands['help'] = {
	aliases: ['?'],
	help: "prints the help message",
	fn: function(args, finished){
		console.log("\nEnter any of the following commands:\n")
		Object.keys(commands).forEach( function(key){
			log_command( key, commands[key].help);
		});
		console.log();
		finished();
	}
};

// - --------------------------------------------
// - Quit this program
// - 
commands['quit'] = {
	aliases: ['q', 'exit', 'bye', 'later' ],
	help: 'exits this program',
	fn: function(args, finished){
		process.exit(0);
	}
};

// - --------------------------------------------
// - Clear the terminal screen
// - 
commands['clear'] = {
	aliases: ['cls', 'c'],
	help: 'clear the terminal screen',
	fn: function(args, finished){
		process.stdout.write('\u001B[2J\u001B[0;0f');
		finished();
	}
}

// - --------------------------------------------
// - Display a bunch of colors
// - 
commands['colors'] = {
	aliases: ['color'],
	help: 'display all the colors',
	fn: function(args, finished){
		for( var i = 0; i < 255; i++ ){
			console.log( clc.xterm(i)(i.toString() + " aaaaaaaaaaaaa"), clc.xterm(10).bgXterm(i)(i.toString() + " ----------------") );
		}
		finished();
	}
}







// - --------------------------------------------
// - setup alias to command object map
// - 
var all_commands = {};
Object.keys(commands).forEach( function(key) {
	// add the main command name
	all_commands[key] = commands[key];

	// add the commands aliases
	commands[key].aliases.forEach( function(alias){
		//BUG: different commands with the same alias
		all_commands[alias] = commands[key];
	});
});


// we'll start running after the inventory has comlpeted loading...
inventory.init( function(error){

	if(error){
		console.log( "Unable to start.")
		console.log(error);
		return;
	}

	run();
});



function run() {

	// - --------------------------------------------
	// - main initalization
	// - 
	// load the inventory
	all_commands['list'].fn(null, function(){

		// give context, output the initial prompt
		all_commands['help'].fn(null, function(){
			prompt();
		});
	})


	// - --------------------------------------------
	// - process user input
	// - 
	process.stdin.on('data', function(input){

		var matches = input.toString().match(/([^\s]+)(.*)/);
		if( matches == null ){
			prompt();
			return;
		}

		var command = matches[1].toLowerCase();
		var args = matches[2].trim().split(/\s+/);

		if( command.length == 0 ){
			prompt();
		}
		else if( !all_commands.hasOwnProperty(command) ){
			log_error("unknown command:", command);
			all_commands["help"].fn(command, function(){prompt();});
		} else {
			all_commands[command].fn(args, function(){prompt();});
		}
	});

}









// - --------------------------------------------
// - output vm inventory
// - 
function log_inventory(vms){
	console.log();
	console.log("Virtual machine inventory: ");
	console.log();
	console.log( "    " + clc.bgXterm(8)(sprintf("%-5s %-20s %s", "ID", "Name", "State     ")) );
	vms.forEach( function(vm) {
		console.log( sprintf("    %-5s %s %s", vm.ItemID, clc.xterm(94)( sprintf("%-20s",vm.DisplayName) ), vm.State) );
	});
	console.log();
}

// - --------------------------------------------
// - outputs the vmshell prompt
// - 
function prompt(){

	var dn = inventory.vms[curr].DisplayName;
	var c_dn = clc.xterm(33).bgXterm(0)(dn);
	
	var state =" [" + inventory.vms[curr].State + "]";
	var c_state = clc.bgXterm(0)(state);

	var remaining = clc.width - dn.length - state.length;
	var spaces = "";
	for( i = 0; i < remaining; i++ ){
		spaces += clc.bgXterm(0)(" ");
	}
	var prompt = clc.bgXterm(0)("  %") + " ";
	console.log(c_dn + c_state + spaces);
	process.stdout.write(prompt);
}

// - --------------------------------------------
// - fancy colored output for error text
// - 
function log_error( prefix, suffix ){
	console.log( clc.red("Error " + prefix) + " " + suffix );
}

// - --------------------------------------------
// - fancy colored output for commands in help string
// - 
function log_command(name, description){
	console.log( "%s   %s", clc.xterm(94)( sprintf("%10s", name ) ), clc.xterm(10)(description) );

}